==> cirq-0.3.1.27-py2-none-any.whl: cirq/testing/equals_tester_test.py <==
# coding=utf-8
# Copyright 2018 The Cirq Developers
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import with_statement
from __future__ import absolute_import
import fractions
import pytest

from cirq.testing.equals_tester import EqualsTester


def test_add_equality_group_correct():
    eq = EqualsTester()

    eq.add_equality_group(fractions.Fraction(1, 1))

    eq.add_equality_group(fractions.Fraction(1, 2), fractions.Fraction(2, 4))

    eq.add_equality_group(
        fractions.Fraction(2, 3),
        fractions.Fraction(12, 18), fractions.Fraction(14, 21))

    eq.add_equality_group(2, 2.0, fractions.Fraction(2, 1))

    eq.add_equality_group([1, 2, 3], [1, 2, 3])

    eq.add_equality_group({u'b': 3, u'a': 2}, {u'a': 2, u'b': 3})

    eq.add_equality_group(u'unrelated')


def test_assert_make_equality_group():
    eq = EqualsTester()

    with pytest.raises(AssertionError):
        eq.make_equality_group(object)

    eq.make_equality_group(lambda: 1)
    eq.make_equality_group(lambda: 2, lambda: 2.0)
    eq.add_equality_group(3)

    with pytest.raises(AssertionError):
        eq.add_equality_group(1)
    with pytest.raises(AssertionError):
        eq.make_equality_group(lambda: 1)
    with pytest.raises(AssertionError):
        eq.make_equality_group(lambda: 3)


def test_add_equality_group_not_equivalent():
    eq = EqualsTester()
    with pytest.raises(AssertionError):
        eq.add_equality_group(1, 2)


def test_add_equality_group_not_disjoint():
    eq = EqualsTester()
    eq.add_equality_group(1)
    with pytest.raises(AssertionError):
        eq.add_equality_group(1)


def test_add_equality_group_bad_hash():
    class KeyHash(object):
        def __init__(self, k, h):
            self._k = k
            self._h = h

        def __eq__(self, other):
            return isinstance(other, KeyHash) and self._k == other._k

        def __ne__(self, other):
            return not self == other

        def __hash__(self):
            return self._h

    eq = EqualsTester()
    eq.add_equality_group(KeyHash(u'a', 5), KeyHash(u'a', 5))
    eq.add_equality_group(KeyHash(u'b', 5))
    with pytest.raises(AssertionError):
        eq.add_equality_group(KeyHash(u'c', 2), KeyHash(u'c', 3))


def test_add_equality_group_exception_hash():
    class FailHash(object):
        def __hash__(self):
            raise ValueError(u'injected failure')

    eq = EqualsTester()
    with pytest.raises(ValueError):
        eq.add_equality_group(FailHash())


def test_can_fail_when_forgot_type_check():
    eq = EqualsTester()

    class NoTypeCheckEqualImplementation(object):
        def __init__(self):
            self.x = 1

        def __eq__(self, other):
            return self.x == other.x

        def __ne__(self, other):
            return not self == other

        def __hash__(self):
            return hash(self.x)

    with pytest.raises(AttributeError):
        eq.add_equality_group(NoTypeCheckEqualImplementation())


def test_fails_hash_is_default_and_inconsistent():
    eq = EqualsTester()

    class DefaultHashImplementation(object):
        __hash__ = object.__hash__

        def __init__(self):
            self.x = 1

        def __eq__(self, other):
            if not isinstance(other, type(self)):
                return NotImplemented
            return self.x == other.x

        def __ne__(self, other):
            return not self == other

    with pytest.raises(AssertionError):
        eq.make_equality_group(DefaultHashImplementation)


def test_fails_when_ne_is_inconsistent():
    eq = EqualsTester()

    class InconsistentNeImplementation(object):
        def __init__(self):
            self.x = 1

        def __eq__(self, other):
            if not isinstance(other, type(self)):
                return NotImplemented
            return self.x == other.x

        def __ne__(self, other):
            return NotImplemented

        def __hash__(self):
            return hash(self.x)

    with pytest.raises(AssertionError):
        eq.make_equality_group(InconsistentNeImplementation)


def test_fails_when_not_reflexive():
    eq = EqualsTester()

    class NotReflexiveImplementation(object):
        def __init__(self):
            self.x = 1

        def __eq__(self, other):
            if other is not self:
                return NotImplemented
            return False

        def __ne__(self, other):
            return not self == other

    with pytest.raises(AssertionError):
        eq.add_equality_group(NotReflexiveImplementation())


def test_fails_when_not_commutative():
    eq = EqualsTester()

    class NotCommutativeImplementation(object):
        def __init__(self, x):
            self.x = x

        def __eq__(self, other):
            if not isinstance(other, type(self)):
                return NotImplemented
            return self.x <= other.x

        def __ne__(self, other):
            return not self == other

    with pytest.raises(AssertionError):
        eq.add_equality_group(NotCommutativeImplementation(0),
                              NotCommutativeImplementation(1))

    with pytest.raises(AssertionError):
        eq.add_equality_group(NotCommutativeImplementation(1),
                              NotCommutativeImplementation(0))

==> cirq-0.5.0-py2-none-any.whl: cirq/testing/equals_tester_test.py <==
# coding=utf-8
# Copyright 2018 The Cirq Developers
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import with_statement
from __future__ import absolute_import
import fractions
import pytest

from cirq.testing.equals_tester import EqualsTester


def test_add_equality_group_correct():
    eq = EqualsTester()

    eq.add_equality_group(fractions.Fraction(1, 1))

    eq.add_equality_group(fractions.Fraction(1, 2), fractions.Fraction(2, 4))

    eq.add_equality_group(
        fractions.Fraction(2, 3),
        fractions.Fraction(12, 18), fractions.Fraction(14, 21))

    eq.add_equality_group(2, 2.0, fractions.Fraction(2, 1))

    eq.add_equality_group([1, 2, 3], [1, 2, 3])

    eq.add_equality_group({u'b': 3, u'a': 2}, {u'a': 2, u'b': 3})

    eq.add_equality_group(u'unrelated')


def test_assert_make_equality_group():
    eq = EqualsTester()

    with pytest.raises(AssertionError, match=u"can't be in the same"):
        eq.make_equality_group(object)

    eq.make_equality_group(lambda: 1)
    eq.make_equality_group(lambda: 2, lambda: 2.0)
    eq.add_equality_group(3)

    with pytest.raises(AssertionError, match=u"can't be in different"):
        eq.add_equality_group(1)
    with pytest.raises(AssertionError, match=u"can't be in different"):
        eq.make_equality_group(lambda: 1)
    with pytest.raises(AssertionError, match=u"can't be in different"):
        eq.make_equality_group(lambda: 3)


def test_add_equality_group_not_equivalent():
    eq = EqualsTester()
    with pytest.raises(AssertionError, match=u"can't be in the same"):
        eq.add_equality_group(1, 2)


def test_add_equality_group_not_disjoint():
    eq = EqualsTester()
    eq.add_equality_group(1)
    with pytest.raises(AssertionError, match=u"can't be in different"):
        eq.add_equality_group(1)


def test_add_equality_group_bad_hash():
    class KeyHash(object):
        def __init__(self, k, h):
            self._k = k
            self._h = h

        def __eq__(self, other):
            return isinstance(other, KeyHash) and self._k == other._k

        def __ne__(self, other):
            return not self == other

        def __hash__(self):
            return self._h

    eq = EqualsTester()
    eq.add_equality_group(KeyHash(u'a', 5), KeyHash(u'a', 5))
    eq.add_equality_group(KeyHash(u'b', 5))
    with pytest.raises(AssertionError, match=u'produced different hashes'):
        eq.add_equality_group(KeyHash(u'c', 2), KeyHash(u'c', 3))


def test_add_equality_group_exception_hash():
    class FailHash(object):
        def __hash__(self):
            raise ValueError(u'injected failure')

    eq = EqualsTester()
    with pytest.raises(ValueError, match=u'injected failure'):
        eq.add_equality_group(FailHash())


def test_fails_when_forgot_type_check():
    eq = EqualsTester()

    class NoTypeCheckEqualImplementation(object):
        def __init__(self):
            self.x = 1

        def __eq__(self, other):
            return self.x == other.x

        def __ne__(self, other):
            return not self == other

        def __hash__(self):
            return hash(self.x)

    with pytest.raises(AttributeError, match=u"has no attribute 'x'"):
        eq.add_equality_group(NoTypeCheckEqualImplementation())


def test_fails_when_equal_to_everything():
    eq = EqualsTester()

    class AllEqual(object):
        __hash__ = None

        def __eq__(self, other):
            return True

        def __ne__(self, other):
            return False

    with pytest.raises(AssertionError, match=u"can't be in different"):
        eq.add_equality_group(AllEqual())


def test_fails_hash_is_default_and_inconsistent():
    eq = EqualsTester()

    class DefaultHashImplementation(object):
        __hash__ = object.__hash__

        def __init__(self):
            self.x = 1

        def __eq__(self, other):
            if not isinstance(other, type(self)):
                return NotImplemented
            return self.x == other.x

        def __ne__(self, other):
            return not self == other

    with pytest.raises(AssertionError, match=u'different hashes'):
        eq.make_equality_group(DefaultHashImplementation)


def test_fails_when_ne_is_inconsistent():
    eq = EqualsTester()

    class InconsistentNeImplementation(object):
        def __init__(self):
            self.x = 1

        def __eq__(self, other):
            if not isinstance(other, type(self)):
                return NotImplemented  # coverage: ignore
            return self.x == other.x

        def __ne__(self, other):
            if not isinstance(other, type(self)):
                return NotImplemented  # coverage: ignore
            return self.x == other.x

        def __hash__(self):
            return hash(self.x)  # coverage: ignore

    with pytest.raises(AssertionError, match=u'inconsistent'):
        eq.make_equality_group(InconsistentNeImplementation)


def test_fails_when_ne_is_inconsistent_due_to_not_implemented():
    eq = EqualsTester()

    class InconsistentNeImplementation(object):
        def __init__(self):
            self.x = 1

        def __eq__(self, other):
            if not isinstance(other, type(self)):
                return NotImplemented
            return self.x == other.x

        def __ne__(self, other):
            return NotImplemented

        def __hash__(self):
            return hash(self.x)

    with pytest.raises(AssertionError, match=u'inconsistent'):
        eq.make_equality_group(InconsistentNeImplementation)


def test_fails_when_not_reflexive():
    eq = EqualsTester()

    class NotReflexiveImplementation(object):
        def __init__(self):
            self.x = 1

        def __eq__(self, other):
            if other is not self:
                return NotImplemented
            return False

        def __ne__(self, other):
            return not self == other

    with pytest.raises(AssertionError, match=u'equal to itself'):
        eq.add_equality_group(NotReflexiveImplementation())


def test_fails_when_not_commutative():
    eq = EqualsTester()

    class NotCommutativeImplementation(object):
        def __init__(self, x):
            self.x = x

        def __eq__(self, other):
            if not isinstance(other, type(self)):
                return NotImplemented
            return self.x <= other.x

        def __ne__(self, other):
            return not self == other

    with pytest.raises(AssertionError, match=u"can't be in the same"):
        eq.add_equality_group(NotCommutativeImplementation(0),
                              NotCommutativeImplementation(1))

    with pytest.raises(AssertionError, match=u"can't be in the same"):
        eq.add_equality_group(NotCommutativeImplementation(1),
                              NotCommutativeImplementation(0))


def test_works_on_types():
    eq = EqualsTester()
    eq.add_equality_group(object)
    eq.add_equality_group(int)
    eq.add_equality_group(object())
